#pragma once

#include <iostream>
#include <vector>
#include <string>
#include <functional>
#include <stdexcept>

// Simple test framework
class TestRunner {
public:
    using TestFunction = std::function<void()>;
    
    static TestRunner& instance() {
        static TestRunner runner;
        return runner;
    }
    
    void addTest(const std::string& name, TestFunction test) {
        tests_.push_back({name, test});
    }
    
    int runAllTests() {
        int passed = 0;
        int failed = 0;
        
        std::cout << "Running " << tests_.size() << " tests...\n\n";
        
        for (const auto& test : tests_) {
            try {
                test.second();
                std::cout << "[PASS] " << test.first << std::endl;
                passed++;
            } catch (const std::exception& e) {
                std::cout << "[FAIL] " << test.first << " - " << e.what() << std::endl;
                failed++;
            } catch (...) {
                std::cout << "[FAIL] " << test.first << " - Unknown error" << std::endl;
                failed++;
            }
        }
        
        std::cout << "\nResults: " << passed << " passed, " << failed << " failed\n";
        return failed;
    }
    
private:
    std::vector<std::pair<std::string, TestFunction>> tests_;
};

// Test assertion macros
#define ASSERT_TRUE(condition) \
    if (!(condition)) { \
        throw std::runtime_error("Assertion failed: " #condition); \
    }

#define ASSERT_FALSE(condition) \
    if (condition) { \
        throw std::runtime_error("Assertion failed: " #condition " should be false"); \
    }

#define ASSERT_EQ(expected, actual) \
    if ((expected) != (actual)) { \
        throw std::runtime_error("Assertion failed: expected != actual"); \
    }

#define ASSERT_STR_EQ(expected, actual) \
    if ((expected) != (actual)) { \
        throw std::runtime_error("Assertion failed: expected '" + std::string(expected) + "' but got '" + std::string(actual) + "'"); \
    }

#define ASSERT_NOT_EMPTY(str) \
    if ((str).empty()) { \
        throw std::runtime_error("Assertion failed: string should not be empty"); \
    }

#define ASSERT_GE(actual, expected) \
    if ((actual) < (expected)) { \
        throw std::runtime_error("Assertion failed: expected " + std::to_string(actual) + " >= " + std::to_string(expected)); \
    }

// Test registration macro
#define TEST(suite, name) \
    void test_##suite##_##name(); \
    struct TestRegistrar_##suite##_##name { \
        TestRegistrar_##suite##_##name() { \
            TestRunner::instance().addTest(#suite "::" #name, test_##suite##_##name); \
        } \
    }; \
    static TestRegistrar_##suite##_##name registrar_##suite##_##name; \
    void test_##suite##_##name()