#include "GetbinClient.hpp"
#include <filesystem>
#include <fstream>
#include <thread>
#include <chrono>

// Test framework declarations from test_main.cpp
class TestRunner {
public:
    static TestRunner& instance();
    void addTest(const std::string& name, std::function<void()> test);
};

#define ASSERT_TRUE(condition) \
    if (!(condition)) { \
        throw std::runtime_error("Assertion failed: " #condition); \
    }

#define ASSERT_FALSE(condition) \
    if (condition) { \
        throw std::runtime_error("Assertion failed: " #condition " should be false"); \
    }

#define ASSERT_EQ(expected, actual) \
    if ((expected) != (actual)) { \
        throw std::runtime_error("Assertion failed: expected != actual"); \
    }

#define ASSERT_STR_EQ(expected, actual) \
    if ((expected) != (actual)) { \
        throw std::runtime_error("Assertion failed: expected '" + std::string(expected) + "' but got '" + std::string(actual) + "'"); \
    }

#define ASSERT_NOT_EMPTY(str) \
    if ((str).empty()) { \
        throw std::runtime_error("Assertion failed: string should not be empty"); \
    }

#define TEST(name) \
    void test_GetbinClient_##name(); \
    void register_GetbinClient_##name() { \
        TestRunner::instance().addTest("GetbinClient::" #name, test_GetbinClient_##name); \
    } \
    void test_GetbinClient_##name()

// Test helper class for GetbinClient testing
class GetbinClientTestHelper {
public:
    static std::filesystem::path createTempDir() {
        auto tempDir = std::filesystem::temp_directory_path() / "getpkg_client_test" / std::to_string(std::time(nullptr));
        std::filesystem::create_directories(tempDir);
        return tempDir;
    }
    
    static void cleanupTempDir(const std::filesystem::path& dir) {
        if (std::filesystem::exists(dir)) {
            std::filesystem::remove_all(dir);
        }
    }
    
    static void createTestFile(const std::filesystem::path& path, const std::string& content) {
        std::ofstream file(path);
        file << content;
        file.close();
    }
    
    static std::string readTestFile(const std::filesystem::path& path) {
        std::ifstream file(path);
        std::string content((std::istreambuf_iterator<char>(file)), std::istreambuf_iterator<char>());
        return content;
    }
    
    // Mock progress callback for testing
    static bool mockProgressCallback(size_t downloaded, size_t total) {
        return true; // Continue download
    }
    
    static bool cancelProgressCallback(size_t downloaded, size_t total) {
        return false; // Cancel download
    }
};

TEST(DefaultConstructor) {
    GetbinClient client;
    
    // Should work with default server
    ASSERT_NOT_EMPTY(client.getLastError()); // Initially empty, but method should exist
}

TEST(MultiServerConstructor) {
    std::vector<std::string> servers = {"server1.com", "server2.com", "server3.com"};
    GetbinClient client(servers);
    
    // Should initialize with multiple servers
    ASSERT_NOT_EMPTY(client.getLastError()); // Method should exist
}

TEST(EmptyServerList) {
    std::vector<std::string> emptyServers;
    GetbinClient client(emptyServers);
    
    // Should handle empty server list gracefully
    ASSERT_NOT_EMPTY(client.getLastError());
}

TEST(NetworkErrorClassification) {
    GetbinClient client;
    
    // Test error message generation
    ASSERT_NOT_EMPTY(client.getNetworkErrorMessage(NetworkError::ConnectionFailed));
    ASSERT_NOT_EMPTY(client.getNetworkErrorMessage(NetworkError::Timeout));
    ASSERT_NOT_EMPTY(client.getNetworkErrorMessage(NetworkError::NotFound));
    ASSERT_NOT_EMPTY(client.getNetworkErrorMessage(NetworkError::Unauthorized));
    ASSERT_NOT_EMPTY(client.getNetworkErrorMessage(NetworkError::ServerError));
}

TEST(UrlBuilding) {
    GetbinClient client;
    
    // Test URL building with different server formats
    // Note: This tests internal functionality, so we test through public methods
    std::string hash;
    
    // Test with different server URL formats
    bool result1 = client.getHash("http://server.com", "test-tool", "x86_64", hash);
    bool result2 = client.getHash("https://server.com", "test-tool", "x86_64", hash);
    bool result3 = client.getHash("server.com", "test-tool", "x86_64", hash);
    
    // These will fail due to network, but should not crash
    ASSERT_FALSE(result1 || result2 || result3); // All should fail gracefully
}

TEST(FindPackageServerWithMultipleServers) {
    std::vector<std::string> servers = {"server1.com", "server2.com", "server3.com"};
    GetbinClient client(servers);
    
    std::string foundServer;
    bool found = client.findPackageServer("nonexistent-tool", "x86_64", foundServer);
    
    // Should not find nonexistent package, but should not crash
    ASSERT_FALSE(found);
    ASSERT_TRUE(foundServer.empty());
}

TEST(MultiServerDownloadFallback) {
    auto tempDir = GetbinClientTestHelper::createTempDir();
    auto outputPath = tempDir / "test-download";
    
    std::vector<std::string> servers = {"invalid-server1.com", "invalid-server2.com", "invalid-server3.com"};
    GetbinClient client(servers);
    
    // Should try all servers and fail gracefully
    bool result = client.download("nonexistent-tool", "x86_64", outputPath.string());
    ASSERT_FALSE(result);
    
    // Should have error message
    ASSERT_NOT_EMPTY(client.getLastError());
    
    GetbinClientTestHelper::cleanupTempDir(tempDir);
}

TEST(ServerSpecificDownload) {
    auto tempDir = GetbinClientTestHelper::createTempDir();
    auto outputPath = tempDir / "test-download";
    
    GetbinClient client;
    
    // Test server-specific download with invalid server
    NetworkError error = client.downloadFromServer("invalid-server.com", "test-tool", "x86_64", 
                                                  outputPath.string());
    
    // Should return appropriate network error
    ASSERT_TRUE(error != NetworkError::None);
    ASSERT_NOT_EMPTY(client.getNetworkErrorMessage(error));
    
    GetbinClientTestHelper::cleanupTempDir(tempDir);
}

TEST(ProgressCallbackHandling) {
    auto tempDir = GetbinClientTestHelper::createTempDir();
    auto outputPath = tempDir / "test-download";
    
    GetbinClient client;
    
    // Test with progress callback that continues
    bool result1 = client.download("test-tool", "x86_64", outputPath.string(), 
                                  GetbinClientTestHelper::mockProgressCallback);
    ASSERT_FALSE(result1); // Will fail due to network, but should handle callback
    
    // Test with progress callback that cancels
    bool result2 = client.download("test-tool", "x86_64", outputPath.string(), 
                                  GetbinClientTestHelper::cancelProgressCallback);
    ASSERT_FALSE(result2); // Should handle cancellation
    
    GetbinClientTestHelper::cleanupTempDir(tempDir);
}

TEST(HashRetrievalMultiServer) {
    std::vector<std::string> servers = {"server1.com", "server2.com"};
    GetbinClient client(servers);
    
    std::string hash;
    
    // Test multi-server hash retrieval
    bool result = client.getHash("test-tool", "x86_64", hash);
    ASSERT_FALSE(result); // Will fail due to network
    ASSERT_TRUE(hash.empty());
    
    // Test server-specific hash retrieval
    bool result2 = client.getHash("server1.com", "test-tool", "x86_64", hash);
    ASSERT_FALSE(result2); // Will fail due to network
    ASSERT_TRUE(hash.empty());
}

TEST(UploadFunctionality) {
    auto tempDir = GetbinClientTestHelper::createTempDir();
    auto testFile = tempDir / "test-archive.tar.gz";
    
    // Create a test file
    GetbinClientTestHelper::createTestFile(testFile, "test archive content");
    
    GetbinClient client;
    std::string outUrl, outHash;
    
    // Test server-specific upload
    bool result1 = client.upload("test-server.com", testFile.string(), outUrl, outHash, "test-token");
    ASSERT_FALSE(result1); // Will fail due to network
    
    // Test backward compatibility upload
    bool result2 = client.upload(testFile.string(), outUrl, outHash, "test-token");
    ASSERT_FALSE(result2); // Will fail due to network
    
    GetbinClientTestHelper::cleanupTempDir(tempDir);
}

TEST(LegacyMethodsCompatibility) {
    GetbinClient client;
    
    // Test legacy delete method
    bool deleteResult = client.deleteObject("test-hash", "test-token");
    ASSERT_FALSE(deleteResult); // Will fail due to network
    
    // Test legacy list packages method
    std::vector<std::string> packages;
    bool listResult = client.listPackages(packages);
    ASSERT_FALSE(listResult); // Will fail due to network
    ASSERT_TRUE(packages.empty());
    
    // Test legacy list all entries method
    std::vector<std::pair<std::string, std::vector<std::string>>> entries;
    bool entriesResult = client.listAllEntries(entries);
    ASSERT_FALSE(entriesResult); // Will fail due to network
    ASSERT_TRUE(entries.empty());
}

TEST(ErrorMessagePersistence) {
    GetbinClient client;
    
    // Trigger an error
    std::string hash;
    client.getHash("invalid-server.com", "test-tool", "x86_64", hash);
    
    // Error message should be set
    std::string error1 = client.getLastError();
    ASSERT_NOT_EMPTY(error1);
    
    // Trigger another error
    client.download("test-tool", "x86_64", "/invalid/path");
    
    // Error message should be updated
    std::string error2 = client.getLastError();
    ASSERT_NOT_EMPTY(error2);
    
    // Errors might be different depending on which fails first
    // But both should be non-empty
}

TEST(UserAgentGeneration) {
    GetbinClient client;
    
    // Test that user agent is properly set (indirect test through network calls)
    std::string hash;
    bool result = client.getHash("test-server.com", "test-tool", "x86_64", hash);
    
    // Should fail due to network but not crash due to user agent issues
    ASSERT_FALSE(result);
    ASSERT_NOT_EMPTY(client.getLastError());
}

TEST(ConcurrentOperations) {
    std::vector<std::string> servers = {"server1.com", "server2.com"};
    GetbinClient client(servers);
    
    // Test that multiple operations can be performed without interference
    std::string hash1, hash2;
    
    bool result1 = client.getHash("tool1", "x86_64", hash1);
    bool result2 = client.getHash("tool2", "aarch64", hash2);
    
    // Both should fail gracefully without interfering with each other
    ASSERT_FALSE(result1);
    ASSERT_FALSE(result2);
    ASSERT_TRUE(hash1.empty());
    ASSERT_TRUE(hash2.empty());
}

TEST(ArchitectureHandling) {
    GetbinClient client;
    
    std::string hash;
    
    // Test different architecture strings
    bool result1 = client.getHash("test-tool", "x86_64", hash);
    bool result2 = client.getHash("test-tool", "aarch64", hash);
    bool result3 = client.getHash("test-tool", "universal", hash);
    bool result4 = client.getHash("test-tool", "invalid-arch", hash);
    
    // All should fail due to network but handle architectures properly
    ASSERT_FALSE(result1);
    ASSERT_FALSE(result2);
    ASSERT_FALSE(result3);
    ASSERT_FALSE(result4);
}

// Registration function
void registerGetbinClientTests() {
    register_GetbinClient_DefaultConstructor();
    register_GetbinClient_MultiServerConstructor();
    register_GetbinClient_EmptyServerList();
    register_GetbinClient_NetworkErrorClassification();
    register_GetbinClient_UrlBuilding();
    register_GetbinClient_FindPackageServerWithMultipleServers();
    register_GetbinClient_MultiServerDownloadFallback();
    register_GetbinClient_ServerSpecificDownload();
    register_GetbinClient_ProgressCallbackHandling();
    register_GetbinClient_HashRetrievalMultiServer();
    register_GetbinClient_UploadFunctionality();
    register_GetbinClient_LegacyMethodsCompatibility();
    register_GetbinClient_ErrorMessagePersistence();
    register_GetbinClient_UserAgentGeneration();
    register_GetbinClient_ConcurrentOperations();
    register_GetbinClient_ArchitectureHandling();
}