#include "ServerManager.hpp"
#include "GetbinClient.hpp"
#include "PackageMetadata.hpp"
#include "MigrationManager.hpp"
#include <filesystem>
#include <fstream>
#include <nlohmann/json.hpp>

// Test framework declarations from test_main.cpp
class TestRunner {
public:
    static TestRunner& instance();
    void addTest(const std::string& name, std::function<void()> test);
};

#define ASSERT_TRUE(condition) \
    if (!(condition)) { \
        throw std::runtime_error("Assertion failed: " #condition); \
    }

#define ASSERT_FALSE(condition) \
    if (condition) { \
        throw std::runtime_error("Assertion failed: " #condition " should be false"); \
    }

#define ASSERT_EQ(expected, actual) \
    if ((expected) != (actual)) { \
        throw std::runtime_error("Assertion failed: expected != actual"); \
    }

#define ASSERT_STR_EQ(expected, actual) \
    if ((expected) != (actual)) { \
        throw std::runtime_error("Assertion failed: expected '" + std::string(expected) + "' but got '" + std::string(actual) + "'"); \
    }

#define ASSERT_NOT_EMPTY(str) \
    if ((str).empty()) { \
        throw std::runtime_error("Assertion failed: string should not be empty"); \
    }

#define ASSERT_GE(actual, expected) \
    if ((actual) < (expected)) { \
        throw std::runtime_error("Assertion failed: expected " + std::to_string(actual) + " >= " + std::to_string(expected)); \
    }

#define TEST(name) \
    void test_Integration_##name(); \
    void register_Integration_##name() { \
        TestRunner::instance().addTest("Integration::" #name, test_Integration_##name); \
    } \
    void test_Integration_##name()

// Test helper class for Integration testing
class IntegrationTestHelper {
public:
    static std::filesystem::path createTempDir() {
        auto tempDir = std::filesystem::temp_directory_path() / "getpkg_integration_test" / std::to_string(std::time(nullptr));
        std::filesystem::create_directories(tempDir);
        return tempDir;
    }
    
    static void cleanupTempDir(const std::filesystem::path& dir) {
        if (std::filesystem::exists(dir)) {
            std::filesystem::remove_all(dir);
        }
    }
    
    static void setupLegacyEnvironment(const std::filesystem::path& configDir) {
        // Create legacy token file
        auto legacyDir = configDir / "getpkg.xyz";
        std::filesystem::create_directories(legacyDir);
        
        std::ofstream tokenFile(legacyDir / "write_token.txt");
        tokenFile << "legacy-integration-token";
        tokenFile.close();
        
        // Create legacy package files
        nlohmann::json package1 = {
            {"name", "integration-tool1"},
            {"version", "2023.1201.1000"},
            {"hash", "legacy123hash456"},
            {"arch", "x86_64"}
        };
        
        nlohmann::json package2 = {
            {"name", "integration-tool2"},
            {"version", "2023.1202.1100"},
            {"hash", "legacy789hash012"},
            {"arch", "aarch64"}
        };
        
        std::ofstream package1File(configDir / "integration-tool1.json");
        package1File << package1.dump(2);
        package1File.close();
        
        std::ofstream package2File(configDir / "integration-tool2.json");
        package2File << package2.dump(2);
        package2File.close();
    }
    
    static void verifyNewFormatStructure(const std::filesystem::path& configDir) {
        ASSERT_TRUE(std::filesystem::exists(configDir / "servers.json"));
        ASSERT_TRUE(std::filesystem::exists(configDir / "packages"));
        ASSERT_TRUE(std::filesystem::is_directory(configDir / "packages"));
    }
    
    static void setEnvironmentHome(const std::filesystem::path& homeDir) {
        setenv("HOME", homeDir.c_str(), 1);
    }
};

TEST(CompleteWorkflowFromLegacyToMultiServer) {
    auto tempDir = IntegrationTestHelper::createTempDir();
    auto configDir = tempDir / ".config" / "getpkg";
    std::filesystem::create_directories(configDir);
    
    // Set up legacy environment
    IntegrationTestHelper::setupLegacyEnvironment(configDir);
    IntegrationTestHelper::setEnvironmentHome(tempDir);
    
    // Step 1: Migration Manager detects need for migration
    MigrationManager migrationManager(configDir);
    ASSERT_TRUE(migrationManager.needsMigration());
    
    // Step 2: Perform migration
    bool migrationResult = migrationManager.performMigration();
    ASSERT_TRUE(migrationResult);
    
    // Verify migration results
    auto result = migrationManager.getLastMigrationResult();
    ASSERT_TRUE(result.success);
    ASSERT_EQ(2, result.migratedPackages);
    ASSERT_TRUE(result.serverConfigMigrated);
    
    // Step 3: ServerManager loads new configuration
    ServerManager serverManager;
    auto loadResult = serverManager.loadConfiguration();
    ASSERT_EQ(ServerManagerError::None, loadResult);
    
    auto servers = serverManager.getServers();
    ASSERT_EQ(1, servers.size());
    ASSERT_STR_EQ("getpkg.xyz", servers[0]);
    ASSERT_TRUE(serverManager.hasWriteToken("getpkg.xyz"));
    
    // Step 4: Add additional servers
    auto addResult = serverManager.addServer("packages.example.com");
    ASSERT_EQ(ServerManagerError::None, addResult);
    
    serverManager.setWriteToken("packages.example.com", "example-token");
    
    // Step 5: GetbinClient uses multi-server configuration
    auto updatedServers = serverManager.getServers();
    GetbinClient client(updatedServers);
    
    // Test multi-server operations (will fail due to network but should not crash)
    std::string hash;
    bool hashResult = client.getHash("test-tool", "x86_64", hash);
    ASSERT_FALSE(hashResult); // Expected to fail due to network
    ASSERT_NOT_EMPTY(client.getLastError());
    
    // Step 6: PackageMetadataManager works with migrated data
    PackageMetadataManager packageManager(configDir);
    
    auto installedPackages = packageManager.listInstalledPackages();
    ASSERT_EQ(2, installedPackages.size());
    
    // Verify migrated packages have server information
    PackageMetadata tool1 = packageManager.loadPackageMetadata("integration-tool1");
    ASSERT_TRUE(tool1.isValid());
    ASSERT_STR_EQ("getpkg.xyz", tool1.sourceServer);
    ASSERT_NOT_EMPTY(tool1.installDate);
    
    // Step 7: Save new package with multi-server metadata
    PackageMetadata newPackage("new-tool", "2024.0115.1430", "new123hash456", "x86_64", "packages.example.com");
    bool saveResult = packageManager.savePackageMetadata(newPackage);
    ASSERT_TRUE(saveResult);
    
    // Verify new package is tracked
    auto updatedPackages = packageManager.listInstalledPackages();
    ASSERT_EQ(3, updatedPackages.size());
    
    IntegrationTestHelper::cleanupTempDir(tempDir);
}

TEST(ServerManagerAndGetbinClientIntegration) {
    auto tempDir = IntegrationTestHelper::createTempDir();
    auto configDir = tempDir / ".config" / "getpkg";
    std::filesystem::create_directories(configDir);
    IntegrationTestHelper::setEnvironmentHome(tempDir);
    
    // Initialize ServerManager
    ServerManager serverManager;
    serverManager.ensureDefaultConfiguration();
    
    // Add multiple servers
    serverManager.addServer("server1.example.com");
    serverManager.addServer("server2.example.com");
    serverManager.setWriteToken("server1.example.com", "token1");
    serverManager.setWriteToken("server2.example.com", "token2");
    
    // Get server list for client
    auto servers = serverManager.getServers();
    ASSERT_EQ(3, servers.size()); // default + 2 added
    
    // Initialize GetbinClient with server list
    GetbinClient client(servers);
    
    // Test server-specific operations
    std::string foundServer;
    bool findResult = client.findPackageServer("test-tool", "x86_64", foundServer);
    ASSERT_FALSE(findResult); // Will fail due to network
    
    // Test fallback behavior
    std::string hash;
    bool hashResult = client.getHash("test-tool", "x86_64", hash);
    ASSERT_FALSE(hashResult); // Will fail but should try all servers
    
    // Verify error handling
    ASSERT_NOT_EMPTY(client.getLastError());
    
    IntegrationTestHelper::cleanupTempDir(tempDir);
}

TEST(PackageMetadataAndMigrationIntegration) {
    auto tempDir = IntegrationTestHelper::createTempDir();
    auto configDir = tempDir / ".config" / "getpkg";
    std::filesystem::create_directories(configDir);
    
    // Create legacy package files
    nlohmann::json legacyPackage1 = {
        {"name", "metadata-tool1"},
        {"version", "1.0"},
        {"hash", "hash1"},
        {"arch", "x86_64"}
    };
    
    nlohmann::json legacyPackage2 = {
        {"name", "metadata-tool2"},
        {"version", "2.0"},
        {"hash", "hash2"},
        {"arch", "aarch64"}
    };
    
    std::ofstream file1(configDir / "metadata-tool1.json");
    file1 << legacyPackage1.dump(2);
    file1.close();
    
    std::ofstream file2(configDir / "metadata-tool2.json");
    file2 << legacyPackage2.dump(2);
    file2.close();
    
    // Use MigrationManager to migrate
    MigrationManager migrationManager(configDir);
    ASSERT_TRUE(migrationManager.needsMigration());
    
    bool migrationResult = migrationManager.performMigration();
    ASSERT_TRUE(migrationResult);
    
    // Use PackageMetadataManager to verify migration
    PackageMetadataManager packageManager(configDir);
    
    auto packages = packageManager.listInstalledPackages();
    ASSERT_EQ(2, packages.size());
    
    // Verify migrated metadata
    PackageMetadata tool1 = packageManager.loadPackageMetadata("metadata-tool1");
    ASSERT_TRUE(tool1.isValid());
    ASSERT_STR_EQ("metadata-tool1", tool1.name);
    ASSERT_STR_EQ("1.0", tool1.version);
    ASSERT_STR_EQ("getpkg.xyz", tool1.sourceServer); // Should be set during migration
    ASSERT_NOT_EMPTY(tool1.installDate);
    
    PackageMetadata tool2 = packageManager.loadPackageMetadata("metadata-tool2");
    ASSERT_TRUE(tool2.isValid());
    ASSERT_STR_EQ("aarch64", tool2.arch);
    ASSERT_STR_EQ("getpkg.xyz", tool2.sourceServer);
    
    IntegrationTestHelper::cleanupTempDir(tempDir);
}

TEST(EndToEndPackageInstallationWorkflow) {
    auto tempDir = IntegrationTestHelper::createTempDir();
    auto configDir = tempDir / ".config" / "getpkg";
    std::filesystem::create_directories(configDir);
    IntegrationTestHelper::setEnvironmentHome(tempDir);
    
    // Step 1: Initialize ServerManager with multiple servers
    ServerManager serverManager;
    serverManager.ensureDefaultConfiguration();
    serverManager.addServer("primary.packages.com");
    serverManager.addServer("backup.packages.com");
    
    // Step 2: Initialize GetbinClient with server list
    auto servers = serverManager.getServers();
    GetbinClient client(servers);
    
    // Step 3: Initialize PackageMetadataManager
    PackageMetadataManager packageManager(configDir);
    packageManager.ensurePackagesDirectory();
    
    // Step 4: Simulate package installation workflow
    std::string toolName = "workflow-test-tool";
    std::string version = "2024.0115.1430";
    std::string hash = "workflow123hash456";
    std::string arch = "x86_64";
    std::string sourceServer = "primary.packages.com";
    
    // Try to download (will fail due to network but tests the workflow)
    auto downloadPath = tempDir / "downloads" / (toolName + ".tar.gz");
    std::filesystem::create_directories(downloadPath.parent_path());
    
    bool downloadResult = client.download(toolName, arch, downloadPath.string());
    ASSERT_FALSE(downloadResult); // Expected to fail due to network
    
    // Simulate successful installation by creating metadata
    PackageMetadata metadata(toolName, version, hash, arch, sourceServer);
    bool saveResult = packageManager.savePackageMetadata(metadata);
    ASSERT_TRUE(saveResult);
    
    // Step 5: Verify package is tracked
    ASSERT_TRUE(packageManager.packageExists(toolName));
    
    PackageMetadata savedMetadata = packageManager.loadPackageMetadata(toolName);
    ASSERT_TRUE(savedMetadata.isValid());
    ASSERT_STR_EQ(sourceServer, savedMetadata.sourceServer);
    
    // Step 6: Simulate update check
    bool needsUpdate = savedMetadata.needsUpdate("different-hash");
    ASSERT_TRUE(needsUpdate);
    
    bool noUpdateNeeded = savedMetadata.needsUpdate(hash);
    ASSERT_FALSE(noUpdateNeeded);
    
    IntegrationTestHelper::cleanupTempDir(tempDir);
}

TEST(MultiServerPublishingWorkflow) {
    auto tempDir = IntegrationTestHelper::createTempDir();
    auto configDir = tempDir / ".config" / "getpkg";
    std::filesystem::create_directories(configDir);
    IntegrationTestHelper::setEnvironmentHome(tempDir);
    
    // Initialize ServerManager with publishing tokens
    ServerManager serverManager;
    serverManager.ensureDefaultConfiguration();
    serverManager.addServer("publish1.example.com");
    serverManager.addServer("publish2.example.com");
    
    serverManager.setWriteToken("getpkg.xyz", "default-token");
    serverManager.setWriteToken("publish1.example.com", "publish1-token");
    serverManager.setWriteToken("publish2.example.com", "publish2-token");
    
    // Test default publish server selection
    std::string defaultPublishServer = serverManager.getDefaultPublishServer();
    ASSERT_STR_EQ("getpkg.xyz", defaultPublishServer); // First server with token
    
    // Test servers with tokens
    auto serversWithTokens = serverManager.getServersWithTokens();
    ASSERT_EQ(3, serversWithTokens.size());
    
    // Initialize GetbinClient for publishing
    auto servers = serverManager.getServers();
    GetbinClient client(servers);
    
    // Create test archive for publishing
    auto testArchive = tempDir / "test-package.tar.gz";
    std::ofstream archiveFile(testArchive);
    archiveFile << "test archive content";
    archiveFile.close();
    
    // Test server-specific publishing (will fail due to network)
    std::string outUrl, outHash;
    bool publishResult = client.upload("publish1.example.com", testArchive.string(), 
                                      outUrl, outHash, "publish1-token");
    ASSERT_FALSE(publishResult); // Expected to fail due to network
    
    // Test default publishing
    bool defaultPublishResult = client.upload(testArchive.string(), outUrl, outHash, "default-token");
    ASSERT_FALSE(defaultPublishResult); // Expected to fail due to network
    
    // Verify error handling
    ASSERT_NOT_EMPTY(client.getLastError());
    
    IntegrationTestHelper::cleanupTempDir(tempDir);
}

TEST(ErrorHandlingAndRecoveryWorkflow) {
    auto tempDir = IntegrationTestHelper::createTempDir();
    auto configDir = tempDir / ".config" / "getpkg";
    std::filesystem::create_directories(configDir);
    
    // Create corrupted configuration
    std::ofstream corruptedConfig(configDir / "servers.json");
    corruptedConfig << "{ invalid json";
    corruptedConfig.close();
    
    // ServerManager should recover from corruption
    ServerManager serverManager;
    auto loadResult = serverManager.loadConfiguration();
    ASSERT_EQ(ServerManagerError::None, loadResult); // Should recover
    
    auto servers = serverManager.getServers();
    ASSERT_EQ(1, servers.size()); // Should have default server
    ASSERT_STR_EQ("getpkg.xyz", servers[0]);
    
    // Create corrupted package metadata
    std::filesystem::create_directories(configDir / "packages");
    std::ofstream corruptedPackage(configDir / "packages" / "corrupted-tool.json");
    corruptedPackage << "{ invalid json";
    corruptedPackage.close();
    
    // PackageMetadataManager should handle corruption
    PackageMetadataManager packageManager(configDir);
    
    PackageMetadata corruptedMetadata = packageManager.loadPackageMetadata("corrupted-tool");
    ASSERT_FALSE(corruptedMetadata.isValid()); // Should fail gracefully
    
    // Cleanup invalid metadata
    int cleanedCount = packageManager.cleanupInvalidMetadata();
    ASSERT_EQ(1, cleanedCount);
    
    // Verify cleanup worked
    ASSERT_FALSE(packageManager.packageExists("corrupted-tool"));
    
    IntegrationTestHelper::cleanupTempDir(tempDir);
}

TEST(BackwardCompatibilityWorkflow) {
    auto tempDir = IntegrationTestHelper::createTempDir();
    auto configDir = tempDir / ".config" / "getpkg";
    std::filesystem::create_directories(configDir);
    IntegrationTestHelper::setEnvironmentHome(tempDir);
    
    // Test that system works without any configuration (backward compatibility)
    ServerManager serverManager;
    serverManager.ensureDefaultConfiguration();
    
    auto servers = serverManager.getServers();
    ASSERT_EQ(1, servers.size());
    ASSERT_STR_EQ("getpkg.xyz", servers[0]);
    
    // GetbinClient should work with default configuration
    GetbinClient defaultClient; // Default constructor
    
    std::string hash;
    bool hashResult = defaultClient.getHash("test-tool", "x86_64", hash);
    ASSERT_FALSE(hashResult); // Will fail due to network
    
    // Multi-server client should also work
    GetbinClient multiClient(servers);
    bool multiHashResult = multiClient.getHash("test-tool", "x86_64", hash);
    ASSERT_FALSE(multiHashResult); // Will fail due to network
    
    // PackageMetadataManager should work with default structure
    PackageMetadataManager packageManager(configDir);
    bool dirResult = packageManager.ensurePackagesDirectory();
    ASSERT_TRUE(dirResult);
    
    // Should be able to save and load packages
    PackageMetadata testPackage("compat-tool", "1.0", "hash", "x86_64", "getpkg.xyz");
    bool saveResult = packageManager.savePackageMetadata(testPackage);
    ASSERT_TRUE(saveResult);
    
    PackageMetadata loadedPackage = packageManager.loadPackageMetadata("compat-tool");
    ASSERT_TRUE(loadedPackage.isValid());
    ASSERT_STR_EQ("compat-tool", loadedPackage.name);
    
    IntegrationTestHelper::cleanupTempDir(tempDir);
}

TEST(ConcurrentOperationsWorkflow) {
    auto tempDir = IntegrationTestHelper::createTempDir();
    auto configDir = tempDir / ".config" / "getpkg";
    std::filesystem::create_directories(configDir);
    IntegrationTestHelper::setEnvironmentHome(tempDir);
    
    // Initialize components
    ServerManager serverManager;
    serverManager.ensureDefaultConfiguration();
    serverManager.addServer("concurrent1.example.com");
    serverManager.addServer("concurrent2.example.com");
    
    auto servers = serverManager.getServers();
    GetbinClient client(servers);
    
    PackageMetadataManager packageManager(configDir);
    packageManager.ensurePackagesDirectory();
    
    // Simulate concurrent operations
    std::vector<std::string> tools = {"tool1", "tool2", "tool3", "tool4", "tool5"};
    
    // Save multiple packages concurrently (simulated)
    for (const auto& tool : tools) {
        PackageMetadata metadata(tool, "1.0", "hash-" + tool, "x86_64", "concurrent1.example.com");
        bool saveResult = packageManager.savePackageMetadata(metadata);
        ASSERT_TRUE(saveResult);
    }
    
    // Verify all packages were saved
    auto installedPackages = packageManager.listInstalledPackages();
    ASSERT_EQ(5, installedPackages.size());
    
    // Test concurrent hash requests (will fail due to network but test concurrency)
    for (const auto& tool : tools) {
        std::string hash;
        bool hashResult = client.getHash(tool, "x86_64", hash);
        ASSERT_FALSE(hashResult); // Expected to fail
    }
    
    // Verify no interference between operations
    for (const auto& tool : tools) {
        PackageMetadata metadata = packageManager.loadPackageMetadata(tool);
        ASSERT_TRUE(metadata.isValid());
        ASSERT_STR_EQ(tool, metadata.name);
    }
    
    IntegrationTestHelper::cleanupTempDir(tempDir);
}

// Registration function
void registerIntegrationTests() {
    register_Integration_CompleteWorkflowFromLegacyToMultiServer();
    register_Integration_ServerManagerAndGetbinClientIntegration();
    register_Integration_PackageMetadataAndMigrationIntegration();
    register_Integration_EndToEndPackageInstallationWorkflow();
    register_Integration_MultiServerPublishingWorkflow();
    register_Integration_ErrorHandlingAndRecoveryWorkflow();
    register_Integration_BackwardCompatibilityWorkflow();
    register_Integration_ConcurrentOperationsWorkflow();
}