#include "MigrationManager.hpp"
#include <filesystem>
#include <fstream>
#include <nlohmann/json.hpp>

// Test framework declarations from test_main.cpp
class TestRunner {
public:
    static TestRunner& instance();
    void addTest(const std::string& name, std::function<void()> test);
};

#define ASSERT_TRUE(condition) \
    if (!(condition)) { \
        throw std::runtime_error("Assertion failed: " #condition); \
    }

#define ASSERT_FALSE(condition) \
    if (condition) { \
        throw std::runtime_error("Assertion failed: " #condition " should be false"); \
    }

#define ASSERT_EQ(expected, actual) \
    if ((expected) != (actual)) { \
        throw std::runtime_error("Assertion failed: expected != actual"); \
    }

#define ASSERT_STR_EQ(expected, actual) \
    if ((expected) != (actual)) { \
        throw std::runtime_error("Assertion failed: expected '" + std::string(expected) + "' but got '" + std::string(actual) + "'"); \
    }

#define ASSERT_NOT_EMPTY(str) \
    if ((str).empty()) { \
        throw std::runtime_error("Assertion failed: string should not be empty"); \
    }

#define ASSERT_GE(actual, expected) \
    if ((actual) < (expected)) { \
        throw std::runtime_error("Assertion failed: expected " + std::to_string(actual) + " >= " + std::to_string(expected)); \
    }

#define TEST(name) \
    void test_MigrationManager_##name(); \
    void register_MigrationManager_##name() { \
        TestRunner::instance().addTest("MigrationManager::" #name, test_MigrationManager_##name); \
    } \
    void test_MigrationManager_##name()

// Test helper class for MigrationManager testing
class MigrationManagerTestHelper {
public:
    static std::filesystem::path createTempDir() {
        auto tempDir = std::filesystem::temp_directory_path() / "getpkg_migration_test" / std::to_string(std::time(nullptr));
        std::filesystem::create_directories(tempDir);
        return tempDir;
    }
    
    static void cleanupTempDir(const std::filesystem::path& dir) {
        if (std::filesystem::exists(dir)) {
            std::filesystem::remove_all(dir);
        }
    }
    
    static void createLegacyTokenFile(const std::filesystem::path& configDir, const std::string& token) {
        auto legacyDir = configDir / "getpkg.xyz";
        std::filesystem::create_directories(legacyDir);
        
        std::ofstream tokenFile(legacyDir / "write_token.txt");
        tokenFile << token;
        tokenFile.close();
    }
    
    static void createLegacyPackageFile(const std::filesystem::path& configDir, const std::string& toolName, const nlohmann::json& content) {
        std::ofstream packageFile(configDir / (toolName + ".json"));
        packageFile << content.dump(2);
        packageFile.close();
    }
    
    static void createNewFormatConfig(const std::filesystem::path& configDir) {
        nlohmann::json config = {
            {"version", "1.0"},
            {"servers", {
                {
                    {"url", "getpkg.xyz"},
                    {"name", "Official getpkg Registry"},
                    {"default", true},
                    {"writeToken", ""},
                    {"added", "2024-01-15T10:30:00Z"}
                }
            }},
            {"lastUpdated", "2024-01-15T10:30:00Z"}
        };
        
        std::ofstream configFile(configDir / "servers.json");
        configFile << config.dump(2);
        configFile.close();
    }
    
    static nlohmann::json createLegacyPackageJson(const std::string& name, const std::string& version, const std::string& hash, const std::string& arch) {
        return nlohmann::json{
            {"name", name},
            {"version", version},
            {"hash", hash},
            {"arch", arch}
        };
    }
    
    static void createPackagesDirectory(const std::filesystem::path& configDir) {
        std::filesystem::create_directories(configDir / "packages");
    }
    
    static bool fileExists(const std::filesystem::path& path) {
        return std::filesystem::exists(path);
    }
    
    static int countFilesInDirectory(const std::filesystem::path& dir, const std::string& extension = "") {
        if (!std::filesystem::exists(dir)) return 0;
        
        int count = 0;
        for (const auto& entry : std::filesystem::directory_iterator(dir)) {
            if (entry.is_regular_file()) {
                if (extension.empty() || entry.path().extension() == extension) {
                    count++;
                }
            }
        }
        return count;
    }
};

TEST(DefaultConstructor) {
    MigrationManager manager;
    
    // Should initialize without crashing
    // Actual functionality depends on environment setup
}

TEST(CustomConstructor) {
    auto tempDir = MigrationManagerTestHelper::createTempDir();
    
    MigrationManager manager(tempDir);
    
    // Should initialize with custom config directory
    MigrationManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(NeedsMigrationNoLegacyData) {
    auto tempDir = MigrationManagerTestHelper::createTempDir();
    
    MigrationManager manager(tempDir);
    
    // No legacy data - should not need migration
    ASSERT_FALSE(manager.needsMigration());
    
    MigrationManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(NeedsMigrationWithLegacyToken) {
    auto tempDir = MigrationManagerTestHelper::createTempDir();
    
    // Create legacy token file
    MigrationManagerTestHelper::createLegacyTokenFile(tempDir, "legacy-token-123");
    
    MigrationManager manager(tempDir);
    
    // Should need migration due to legacy token
    ASSERT_TRUE(manager.needsMigration());
    
    MigrationManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(NeedsMigrationWithLegacyPackages) {
    auto tempDir = MigrationManagerTestHelper::createTempDir();
    
    // Create legacy package files
    auto legacyPackage = MigrationManagerTestHelper::createLegacyPackageJson("test-tool", "1.0", "hash123", "x86_64");
    MigrationManagerTestHelper::createLegacyPackageFile(tempDir, "test-tool", legacyPackage);
    
    MigrationManager manager(tempDir);
    
    // Should need migration due to legacy packages
    ASSERT_TRUE(manager.needsMigration());
    
    MigrationManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(NeedsMigrationWithNewFormat) {
    auto tempDir = MigrationManagerTestHelper::createTempDir();
    
    // Create new format configuration
    MigrationManagerTestHelper::createNewFormatConfig(tempDir);
    MigrationManagerTestHelper::createPackagesDirectory(tempDir);
    
    MigrationManager manager(tempDir);
    
    // Should not need migration - already in new format
    ASSERT_FALSE(manager.needsMigration());
    
    MigrationManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(MigrateServerConfigurationSuccess) {
    auto tempDir = MigrationManagerTestHelper::createTempDir();
    
    // Create legacy token file
    MigrationManagerTestHelper::createLegacyTokenFile(tempDir, "legacy-token-456");
    
    MigrationManager manager(tempDir);
    
    // Migrate server configuration
    bool result = manager.migrateServerConfiguration();
    ASSERT_TRUE(result);
    
    // Verify servers.json was created
    ASSERT_TRUE(MigrationManagerTestHelper::fileExists(tempDir / "servers.json"));
    
    MigrationManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(MigratePackageMetadataSuccess) {
    auto tempDir = MigrationManagerTestHelper::createTempDir();
    
    // Create legacy package files
    auto package1 = MigrationManagerTestHelper::createLegacyPackageJson("tool1", "1.0", "hash1", "x86_64");
    auto package2 = MigrationManagerTestHelper::createLegacyPackageJson("tool2", "2.0", "hash2", "aarch64");
    
    MigrationManagerTestHelper::createLegacyPackageFile(tempDir, "tool1", package1);
    MigrationManagerTestHelper::createLegacyPackageFile(tempDir, "tool2", package2);
    
    MigrationManager manager(tempDir);
    
    // Migrate package metadata
    bool result = manager.migratePackageMetadata();
    ASSERT_TRUE(result);
    
    // Verify packages directory was created
    ASSERT_TRUE(MigrationManagerTestHelper::fileExists(tempDir / "packages"));
    
    // Verify package files were moved and updated
    ASSERT_TRUE(MigrationManagerTestHelper::fileExists(tempDir / "packages" / "tool1.json"));
    ASSERT_TRUE(MigrationManagerTestHelper::fileExists(tempDir / "packages" / "tool2.json"));
    
    MigrationManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(CreatePackagesDirectorySuccess) {
    auto tempDir = MigrationManagerTestHelper::createTempDir();
    
    MigrationManager manager(tempDir);
    
    // Create packages directory
    bool result = manager.createPackagesDirectory();
    ASSERT_TRUE(result);
    
    // Verify directory was created
    ASSERT_TRUE(MigrationManagerTestHelper::fileExists(tempDir / "packages"));
    ASSERT_TRUE(std::filesystem::is_directory(tempDir / "packages"));
    
    MigrationManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(PerformFullMigrationSuccess) {
    auto tempDir = MigrationManagerTestHelper::createTempDir();
    
    // Set up legacy environment
    MigrationManagerTestHelper::createLegacyTokenFile(tempDir, "full-migration-token");
    
    auto package1 = MigrationManagerTestHelper::createLegacyPackageJson("migrate-tool1", "1.0", "hash1", "x86_64");
    auto package2 = MigrationManagerTestHelper::createLegacyPackageJson("migrate-tool2", "2.0", "hash2", "aarch64");
    
    MigrationManagerTestHelper::createLegacyPackageFile(tempDir, "migrate-tool1", package1);
    MigrationManagerTestHelper::createLegacyPackageFile(tempDir, "migrate-tool2", package2);
    
    MigrationManager manager(tempDir);
    
    // Verify migration is needed
    ASSERT_TRUE(manager.needsMigration());
    
    // Perform full migration
    bool result = manager.performMigration();
    ASSERT_TRUE(result);
    
    // Verify migration results
    auto migrationResult = manager.getLastMigrationResult();
    ASSERT_TRUE(migrationResult.success);
    ASSERT_EQ(2, migrationResult.migratedPackages);
    ASSERT_EQ(2, migrationResult.totalPackages);
    ASSERT_TRUE(migrationResult.serverConfigMigrated);
    ASSERT_TRUE(migrationResult.packageDirectoryCreated);
    
    // Verify new format files exist
    ASSERT_TRUE(MigrationManagerTestHelper::fileExists(tempDir / "servers.json"));
    ASSERT_TRUE(MigrationManagerTestHelper::fileExists(tempDir / "packages"));
    ASSERT_TRUE(MigrationManagerTestHelper::fileExists(tempDir / "packages" / "migrate-tool1.json"));
    ASSERT_TRUE(MigrationManagerTestHelper::fileExists(tempDir / "packages" / "migrate-tool2.json"));
    
    // Verify no longer needs migration
    ASSERT_FALSE(manager.needsMigration());
    
    MigrationManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(PerformMigrationNoLegacyData) {
    auto tempDir = MigrationManagerTestHelper::createTempDir();
    
    MigrationManager manager(tempDir);
    
    // Should not need migration
    ASSERT_FALSE(manager.needsMigration());
    
    // Migration should succeed but do nothing
    bool result = manager.performMigration();
    ASSERT_TRUE(result);
    
    auto migrationResult = manager.getLastMigrationResult();
    ASSERT_TRUE(migrationResult.success);
    ASSERT_EQ(0, migrationResult.migratedPackages);
    ASSERT_EQ(0, migrationResult.totalPackages);
    
    MigrationManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(ValidateMigrationSuccess) {
    auto tempDir = MigrationManagerTestHelper::createTempDir();
    
    // Create proper new format structure
    MigrationManagerTestHelper::createNewFormatConfig(tempDir);
    MigrationManagerTestHelper::createPackagesDirectory(tempDir);
    
    MigrationManager manager(tempDir);
    
    // Validation should pass
    bool result = manager.validateMigration();
    ASSERT_TRUE(result);
    
    MigrationManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(ValidateMigrationFailure) {
    auto tempDir = MigrationManagerTestHelper::createTempDir();
    
    // Create incomplete structure (missing packages directory)
    MigrationManagerTestHelper::createNewFormatConfig(tempDir);
    // Don't create packages directory
    
    MigrationManager manager(tempDir);
    
    // Validation should fail
    bool result = manager.validateMigration();
    ASSERT_FALSE(result);
    
    MigrationManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(CreateBackupSuccess) {
    auto tempDir = MigrationManagerTestHelper::createTempDir();
    
    // Create some files to backup
    MigrationManagerTestHelper::createLegacyTokenFile(tempDir, "backup-token");
    auto package = MigrationManagerTestHelper::createLegacyPackageJson("backup-tool", "1.0", "hash", "x86_64");
    MigrationManagerTestHelper::createLegacyPackageFile(tempDir, "backup-tool", package);
    
    MigrationManager manager(tempDir);
    
    // Create backup
    bool result = manager.createBackup();
    ASSERT_TRUE(result);
    
    // Verify backup directory exists
    ASSERT_TRUE(MigrationManagerTestHelper::fileExists(tempDir / "migration_backup"));
    
    MigrationManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(RollbackCapability) {
    auto tempDir = MigrationManagerTestHelper::createTempDir();
    
    // Set up legacy environment
    MigrationManagerTestHelper::createLegacyTokenFile(tempDir, "rollback-token");
    auto package = MigrationManagerTestHelper::createLegacyPackageJson("rollback-tool", "1.0", "hash", "x86_64");
    MigrationManagerTestHelper::createLegacyPackageFile(tempDir, "rollback-tool", package);
    
    MigrationManager manager(tempDir);
    
    // Initially should not be able to rollback
    ASSERT_FALSE(manager.canRollback());
    
    // Create backup
    manager.createBackup();
    
    // Now should be able to rollback
    ASSERT_TRUE(manager.canRollback());
    
    // Perform migration
    manager.performMigration();
    
    // Should still be able to rollback
    ASSERT_TRUE(manager.canRollback());
    
    MigrationManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(PerformRollbackSuccess) {
    auto tempDir = MigrationManagerTestHelper::createTempDir();
    
    // Set up legacy environment
    MigrationManagerTestHelper::createLegacyTokenFile(tempDir, "rollback-test-token");
    auto package = MigrationManagerTestHelper::createLegacyPackageJson("rollback-test-tool", "1.0", "hash", "x86_64");
    MigrationManagerTestHelper::createLegacyPackageFile(tempDir, "rollback-test-tool", package);
    
    MigrationManager manager(tempDir);
    
    // Create backup and perform migration
    manager.createBackup();
    manager.performMigration();
    
    // Verify migration completed
    ASSERT_TRUE(MigrationManagerTestHelper::fileExists(tempDir / "servers.json"));
    ASSERT_TRUE(MigrationManagerTestHelper::fileExists(tempDir / "packages"));
    
    // Perform rollback
    bool rollbackResult = manager.performRollback();
    ASSERT_TRUE(rollbackResult);
    
    // Verify rollback restored original state
    ASSERT_TRUE(MigrationManagerTestHelper::fileExists(tempDir / "getpkg.xyz" / "write_token.txt"));
    ASSERT_TRUE(MigrationManagerTestHelper::fileExists(tempDir / "rollback-test-tool.json"));
    
    MigrationManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(MigrationWithCorruptedLegacyData) {
    auto tempDir = MigrationManagerTestHelper::createTempDir();
    
    // Create corrupted legacy package file
    std::ofstream corruptedFile(tempDir / "corrupted-tool.json");
    corruptedFile << "{ invalid json content";
    corruptedFile.close();
    
    // Create valid legacy data too
    MigrationManagerTestHelper::createLegacyTokenFile(tempDir, "valid-token");
    auto validPackage = MigrationManagerTestHelper::createLegacyPackageJson("valid-tool", "1.0", "hash", "x86_64");
    MigrationManagerTestHelper::createLegacyPackageFile(tempDir, "valid-tool", validPackage);
    
    MigrationManager manager(tempDir);
    
    // Should still need migration
    ASSERT_TRUE(manager.needsMigration());
    
    // Migration should handle corrupted data gracefully
    bool result = manager.performMigration();
    ASSERT_TRUE(result); // Should succeed despite corrupted data
    
    auto migrationResult = manager.getLastMigrationResult();
    ASSERT_TRUE(migrationResult.success);
    ASSERT_GE(migrationResult.errors.size(), 0); // May have errors for corrupted data
    ASSERT_GE(migrationResult.warnings.size(), 0); // May have warnings
    
    // Valid data should be migrated
    ASSERT_TRUE(MigrationManagerTestHelper::fileExists(tempDir / "packages" / "valid-tool.json"));
    
    MigrationManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(MigrationResultReporting) {
    auto tempDir = MigrationManagerTestHelper::createTempDir();
    
    // Set up test environment with multiple packages
    MigrationManagerTestHelper::createLegacyTokenFile(tempDir, "result-test-token");
    
    for (int i = 1; i <= 3; i++) {
        auto package = MigrationManagerTestHelper::createLegacyPackageJson(
            "result-tool" + std::to_string(i), "1.0", "hash" + std::to_string(i), "x86_64");
        MigrationManagerTestHelper::createLegacyPackageFile(tempDir, "result-tool" + std::to_string(i), package);
    }
    
    MigrationManager manager(tempDir);
    
    // Perform migration
    bool result = manager.performMigration();
    ASSERT_TRUE(result);
    
    // Check detailed results
    auto migrationResult = manager.getLastMigrationResult();
    ASSERT_TRUE(migrationResult.success);
    ASSERT_EQ(3, migrationResult.migratedPackages);
    ASSERT_EQ(3, migrationResult.totalPackages);
    ASSERT_TRUE(migrationResult.serverConfigMigrated);
    ASSERT_TRUE(migrationResult.packageDirectoryCreated);
    
    MigrationManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(MigrationWithExistingNewFormatData) {
    auto tempDir = MigrationManagerTestHelper::createTempDir();
    
    // Create both legacy and new format data
    MigrationManagerTestHelper::createLegacyTokenFile(tempDir, "mixed-token");
    auto legacyPackage = MigrationManagerTestHelper::createLegacyPackageJson("legacy-mixed", "1.0", "hash", "x86_64");
    MigrationManagerTestHelper::createLegacyPackageFile(tempDir, "legacy-mixed", legacyPackage);
    
    // Also create new format data
    MigrationManagerTestHelper::createNewFormatConfig(tempDir);
    MigrationManagerTestHelper::createPackagesDirectory(tempDir);
    
    MigrationManager manager(tempDir);
    
    // Should still need migration due to legacy data
    ASSERT_TRUE(manager.needsMigration());
    
    // Migration should handle mixed environment
    bool result = manager.performMigration();
    ASSERT_TRUE(result);
    
    // Should preserve existing new format data and add legacy data
    ASSERT_TRUE(MigrationManagerTestHelper::fileExists(tempDir / "servers.json"));
    ASSERT_TRUE(MigrationManagerTestHelper::fileExists(tempDir / "packages"));
    ASSERT_TRUE(MigrationManagerTestHelper::fileExists(tempDir / "packages" / "legacy-mixed.json"));
    
    MigrationManagerTestHelper::cleanupTempDir(tempDir);
}

// Registration function
void registerMigrationManagerTests() {
    register_MigrationManager_DefaultConstructor();
    register_MigrationManager_CustomConstructor();
    register_MigrationManager_NeedsMigrationNoLegacyData();
    register_MigrationManager_NeedsMigrationWithLegacyToken();
    register_MigrationManager_NeedsMigrationWithLegacyPackages();
    register_MigrationManager_NeedsMigrationWithNewFormat();
    register_MigrationManager_MigrateServerConfigurationSuccess();
    register_MigrationManager_MigratePackageMetadataSuccess();
    register_MigrationManager_CreatePackagesDirectorySuccess();
    register_MigrationManager_PerformFullMigrationSuccess();
    register_MigrationManager_PerformMigrationNoLegacyData();
    register_MigrationManager_ValidateMigrationSuccess();
    register_MigrationManager_ValidateMigrationFailure();
    register_MigrationManager_CreateBackupSuccess();
    register_MigrationManager_RollbackCapability();
    register_MigrationManager_PerformRollbackSuccess();
    register_MigrationManager_MigrationWithCorruptedLegacyData();
    register_MigrationManager_MigrationResultReporting();
    register_MigrationManager_MigrationWithExistingNewFormatData();
}