#include "PackageMetadata.hpp"
#include <filesystem>
#include <fstream>
#include <nlohmann/json.hpp>

// Test framework declarations from test_main.cpp
class TestRunner {
public:
    static TestRunner& instance();
    void addTest(const std::string& name, std::function<void()> test);
};

#define ASSERT_TRUE(condition) \
    if (!(condition)) { \
        throw std::runtime_error("Assertion failed: " #condition); \
    }

#define ASSERT_FALSE(condition) \
    if (condition) { \
        throw std::runtime_error("Assertion failed: " #condition " should be false"); \
    }

#define ASSERT_EQ(expected, actual) \
    if ((expected) != (actual)) { \
        throw std::runtime_error("Assertion failed: expected != actual"); \
    }

#define ASSERT_STR_EQ(expected, actual) \
    if ((expected) != (actual)) { \
        throw std::runtime_error("Assertion failed: expected '" + std::string(expected) + "' but got '" + std::string(actual) + "'"); \
    }

#define ASSERT_NOT_EMPTY(str) \
    if ((str).empty()) { \
        throw std::runtime_error("Assertion failed: string should not be empty"); \
    }

#define TEST(name) \
    void test_PackageMetadata_##name(); \
    void register_PackageMetadata_##name() { \
        TestRunner::instance().addTest("PackageMetadata::" #name, test_PackageMetadata_##name); \
    } \
    void test_PackageMetadata_##name()

// Test helper class for PackageMetadata testing
class PackageMetadataTestHelper {
public:
    static std::filesystem::path createTempDir() {
        auto tempDir = std::filesystem::temp_directory_path() / "getpkg_metadata_test" / std::to_string(std::time(nullptr));
        std::filesystem::create_directories(tempDir);
        return tempDir;
    }
    
    static void cleanupTempDir(const std::filesystem::path& dir) {
        if (std::filesystem::exists(dir)) {
            std::filesystem::remove_all(dir);
        }
    }
    
    static void createJsonFile(const std::filesystem::path& path, const nlohmann::json& content) {
        std::ofstream file(path);
        file << content.dump(2);
        file.close();
    }
    
    static nlohmann::json readJsonFile(const std::filesystem::path& path) {
        std::ifstream file(path);
        nlohmann::json j;
        file >> j;
        return j;
    }
    
    static PackageMetadata createValidMetadata() {
        return PackageMetadata("test-tool", "2024.0115.1430", "abc123hash456", "x86_64", "getpkg.xyz", "2024-01-15T14:30:00Z");
    }
    
    static nlohmann::json createLegacyJson() {
        return nlohmann::json{
            {"name", "legacy-tool"},
            {"version", "2023.1201.1000"},
            {"hash", "legacy123hash456"},
            {"arch", "x86_64"}
            // Note: no sourceServer or installDate fields
        };
    }
    
    static nlohmann::json createCorruptedJson() {
        return nlohmann::json{
            {"name", ""},  // Invalid empty name
            {"version", "invalid-version-format"},
            {"hash", "short"},  // Too short hash
            {"arch", "invalid-arch"},
            {"sourceServer", "not-a-url"},
            {"installDate", "invalid-date-format"}
        };
    }
};

TEST(DefaultConstructor) {
    PackageMetadata metadata;
    
    ASSERT_TRUE(metadata.name.empty());
    ASSERT_TRUE(metadata.version.empty());
    ASSERT_TRUE(metadata.hash.empty());
    ASSERT_TRUE(metadata.arch.empty());
    ASSERT_TRUE(metadata.sourceServer.empty());
    ASSERT_TRUE(metadata.installDate.empty());
    
    ASSERT_FALSE(metadata.isValid());
}

TEST(ParameterizedConstructor) {
    PackageMetadata metadata("test-tool", "2024.0115.1430", "abc123hash456", "x86_64", "getpkg.xyz", "2024-01-15T14:30:00Z");
    
    ASSERT_STR_EQ("test-tool", metadata.name);
    ASSERT_STR_EQ("2024.0115.1430", metadata.version);
    ASSERT_STR_EQ("abc123hash456", metadata.hash);
    ASSERT_STR_EQ("x86_64", metadata.arch);
    ASSERT_STR_EQ("getpkg.xyz", metadata.sourceServer);
    ASSERT_STR_EQ("2024-01-15T14:30:00Z", metadata.installDate);
    
    ASSERT_TRUE(metadata.isValid());
}

TEST(JsonSerialization) {
    PackageMetadata original = PackageMetadataTestHelper::createValidMetadata();
    
    nlohmann::json j = original.toJson();
    
    ASSERT_STR_EQ("test-tool", j["name"]);
    ASSERT_STR_EQ("2024.0115.1430", j["version"]);
    ASSERT_STR_EQ("abc123hash456", j["hash"]);
    ASSERT_STR_EQ("x86_64", j["arch"]);
    ASSERT_STR_EQ("getpkg.xyz", j["sourceServer"]);
    ASSERT_STR_EQ("2024-01-15T14:30:00Z", j["installDate"]);
}

TEST(JsonDeserialization) {
    nlohmann::json j = {
        {"name", "deserialized-tool"},
        {"version", "2024.0116.0900"},
        {"hash", "def456hash789"},
        {"arch", "aarch64"},
        {"sourceServer", "packages.example.com"},
        {"installDate", "2024-01-16T09:00:00Z"}
    };
    
    PackageMetadata metadata = PackageMetadata::fromJson(j);
    
    ASSERT_STR_EQ("deserialized-tool", metadata.name);
    ASSERT_STR_EQ("2024.0116.0900", metadata.version);
    ASSERT_STR_EQ("def456hash789", metadata.hash);
    ASSERT_STR_EQ("aarch64", metadata.arch);
    ASSERT_STR_EQ("packages.example.com", metadata.sourceServer);
    ASSERT_STR_EQ("2024-01-16T09:00:00Z", metadata.installDate);
    
    ASSERT_TRUE(metadata.isValid());
}

TEST(LegacyJsonMigration) {
    nlohmann::json legacyJson = PackageMetadataTestHelper::createLegacyJson();
    
    PackageMetadata metadata = PackageMetadata::fromLegacyJson(legacyJson, "getpkg.xyz");
    
    ASSERT_STR_EQ("legacy-tool", metadata.name);
    ASSERT_STR_EQ("2023.1201.1000", metadata.version);
    ASSERT_STR_EQ("legacy123hash456", metadata.hash);
    ASSERT_STR_EQ("x86_64", metadata.arch);
    ASSERT_STR_EQ("getpkg.xyz", metadata.sourceServer);
    ASSERT_NOT_EMPTY(metadata.installDate); // Should be auto-generated
    
    ASSERT_TRUE(metadata.isValid());
}

TEST(LegacyJsonMigrationWithCustomServer) {
    nlohmann::json legacyJson = PackageMetadataTestHelper::createLegacyJson();
    
    PackageMetadata metadata = PackageMetadata::fromLegacyJson(legacyJson, "custom.server.com");
    
    ASSERT_STR_EQ("custom.server.com", metadata.sourceServer);
    ASSERT_TRUE(metadata.isValid());
}

TEST(ValidationValid) {
    PackageMetadata metadata = PackageMetadataTestHelper::createValidMetadata();
    
    ASSERT_TRUE(metadata.isValid());
    ASSERT_TRUE(metadata.getValidationError().empty());
}

TEST(ValidationInvalidName) {
    PackageMetadata metadata("", "2024.0115.1430", "abc123hash456", "x86_64", "getpkg.xyz");
    
    ASSERT_FALSE(metadata.isValid());
    ASSERT_NOT_EMPTY(metadata.getValidationError());
}

TEST(ValidationInvalidVersion) {
    PackageMetadata metadata("test-tool", "", "abc123hash456", "x86_64", "getpkg.xyz");
    
    ASSERT_FALSE(metadata.isValid());
    ASSERT_NOT_EMPTY(metadata.getValidationError());
}

TEST(ValidationInvalidHash) {
    PackageMetadata metadata("test-tool", "2024.0115.1430", "", "x86_64", "getpkg.xyz");
    
    ASSERT_FALSE(metadata.isValid());
    ASSERT_NOT_EMPTY(metadata.getValidationError());
}

TEST(ValidationInvalidArch) {
    PackageMetadata metadata("test-tool", "2024.0115.1430", "abc123hash456", "", "getpkg.xyz");
    
    ASSERT_FALSE(metadata.isValid());
    ASSERT_NOT_EMPTY(metadata.getValidationError());
}

TEST(ValidationInvalidServer) {
    PackageMetadata metadata("test-tool", "2024.0115.1430", "abc123hash456", "x86_64", "");
    
    ASSERT_FALSE(metadata.isValid());
    ASSERT_NOT_EMPTY(metadata.getValidationError());
}

TEST(FileOperationsSaveAndLoad) {
    auto tempDir = PackageMetadataTestHelper::createTempDir();
    auto filePath = tempDir / "test-metadata.json";
    
    PackageMetadata original = PackageMetadataTestHelper::createValidMetadata();
    
    // Save to file
    bool saveResult = original.saveToFile(filePath);
    ASSERT_TRUE(saveResult);
    ASSERT_TRUE(std::filesystem::exists(filePath));
    
    // Load from file
    PackageMetadata loaded = PackageMetadata::loadFromFile(filePath);
    
    ASSERT_STR_EQ(original.name, loaded.name);
    ASSERT_STR_EQ(original.version, loaded.version);
    ASSERT_STR_EQ(original.hash, loaded.hash);
    ASSERT_STR_EQ(original.arch, loaded.arch);
    ASSERT_STR_EQ(original.sourceServer, loaded.sourceServer);
    ASSERT_STR_EQ(original.installDate, loaded.installDate);
    
    PackageMetadataTestHelper::cleanupTempDir(tempDir);
}

TEST(FileOperationsInvalidPath) {
    PackageMetadata metadata = PackageMetadataTestHelper::createValidMetadata();
    
    // Try to save to invalid path
    bool saveResult = metadata.saveToFile("/invalid/path/metadata.json");
    ASSERT_FALSE(saveResult);
    
    // Try to load from non-existent file
    PackageMetadata loaded = PackageMetadata::loadFromFile("/nonexistent/file.json");
    ASSERT_FALSE(loaded.isValid());
}

TEST(NeedsUpdateComparison) {
    PackageMetadata metadata = PackageMetadataTestHelper::createValidMetadata();
    
    // Same hash - no update needed
    ASSERT_FALSE(metadata.needsUpdate("abc123hash456"));
    
    // Different hash - update needed
    ASSERT_TRUE(metadata.needsUpdate("different123hash456"));
    
    // Empty hash - update needed
    ASSERT_TRUE(metadata.needsUpdate(""));
}

TEST(TimestampGeneration) {
    PackageMetadata metadata;
    
    std::string timestamp = metadata.getCurrentTimestamp();
    ASSERT_NOT_EMPTY(timestamp);
    
    // Should be in ISO format (basic check)
    ASSERT_TRUE(timestamp.find("T") != std::string::npos);
    ASSERT_TRUE(timestamp.find("Z") != std::string::npos);
}

// PackageMetadataManager Tests

TEST(ManagerDefaultConstructor) {
    PackageMetadataManager manager;
    
    // Should initialize with default config directory
    auto packagesDir = manager.getPackagesDirectory();
    ASSERT_FALSE(packagesDir.empty());
}

TEST(ManagerCustomConstructor) {
    auto tempDir = PackageMetadataTestHelper::createTempDir();
    
    PackageMetadataManager manager(tempDir);
    
    auto packagesDir = manager.getPackagesDirectory();
    ASSERT_STR_EQ((tempDir / "packages").string(), packagesDir.string());
    
    PackageMetadataTestHelper::cleanupTempDir(tempDir);
}

TEST(ManagerEnsurePackagesDirectory) {
    auto tempDir = PackageMetadataTestHelper::createTempDir();
    
    PackageMetadataManager manager(tempDir);
    
    bool result = manager.ensurePackagesDirectory();
    ASSERT_TRUE(result);
    
    auto packagesDir = manager.getPackagesDirectory();
    ASSERT_TRUE(std::filesystem::exists(packagesDir));
    ASSERT_TRUE(std::filesystem::is_directory(packagesDir));
    
    PackageMetadataTestHelper::cleanupTempDir(tempDir);
}

TEST(ManagerSaveAndLoadPackageMetadata) {
    auto tempDir = PackageMetadataTestHelper::createTempDir();
    
    PackageMetadataManager manager(tempDir);
    manager.ensurePackagesDirectory();
    
    PackageMetadata metadata = PackageMetadataTestHelper::createValidMetadata();
    
    // Save metadata
    bool saveResult = manager.savePackageMetadata(metadata);
    ASSERT_TRUE(saveResult);
    
    // Check if package exists
    ASSERT_TRUE(manager.packageExists("test-tool"));
    
    // Load metadata
    PackageMetadata loaded = manager.loadPackageMetadata("test-tool");
    ASSERT_TRUE(loaded.isValid());
    ASSERT_STR_EQ(metadata.name, loaded.name);
    ASSERT_STR_EQ(metadata.version, loaded.version);
    
    PackageMetadataTestHelper::cleanupTempDir(tempDir);
}

TEST(ManagerRemovePackageMetadata) {
    auto tempDir = PackageMetadataTestHelper::createTempDir();
    
    PackageMetadataManager manager(tempDir);
    manager.ensurePackagesDirectory();
    
    PackageMetadata metadata = PackageMetadataTestHelper::createValidMetadata();
    manager.savePackageMetadata(metadata);
    
    ASSERT_TRUE(manager.packageExists("test-tool"));
    
    bool removeResult = manager.removePackageMetadata("test-tool");
    ASSERT_TRUE(removeResult);
    
    ASSERT_FALSE(manager.packageExists("test-tool"));
    
    PackageMetadataTestHelper::cleanupTempDir(tempDir);
}

TEST(ManagerListInstalledPackages) {
    auto tempDir = PackageMetadataTestHelper::createTempDir();
    
    PackageMetadataManager manager(tempDir);
    manager.ensurePackagesDirectory();
    
    // Save multiple packages
    PackageMetadata metadata1("tool1", "1.0", "hash1", "x86_64", "server1");
    PackageMetadata metadata2("tool2", "2.0", "hash2", "aarch64", "server2");
    
    manager.savePackageMetadata(metadata1);
    manager.savePackageMetadata(metadata2);
    
    auto packages = manager.listInstalledPackages();
    ASSERT_EQ(2, packages.size());
    
    // Should contain both tools (order may vary)
    bool foundTool1 = std::find(packages.begin(), packages.end(), "tool1") != packages.end();
    bool foundTool2 = std::find(packages.begin(), packages.end(), "tool2") != packages.end();
    
    ASSERT_TRUE(foundTool1);
    ASSERT_TRUE(foundTool2);
    
    PackageMetadataTestHelper::cleanupTempDir(tempDir);
}

TEST(ManagerGetAllPackageMetadata) {
    auto tempDir = PackageMetadataTestHelper::createTempDir();
    
    PackageMetadataManager manager(tempDir);
    manager.ensurePackagesDirectory();
    
    // Save multiple packages
    PackageMetadata metadata1("tool1", "1.0", "hash1", "x86_64", "server1");
    PackageMetadata metadata2("tool2", "2.0", "hash2", "aarch64", "server2");
    
    manager.savePackageMetadata(metadata1);
    manager.savePackageMetadata(metadata2);
    
    auto allMetadata = manager.getAllPackageMetadata();
    ASSERT_EQ(2, allMetadata.size());
    
    // Verify metadata content
    bool foundTool1 = false, foundTool2 = false;
    for (const auto& meta : allMetadata) {
        if (meta.name == "tool1") {
            foundTool1 = true;
            ASSERT_STR_EQ("1.0", meta.version);
        } else if (meta.name == "tool2") {
            foundTool2 = true;
            ASSERT_STR_EQ("2.0", meta.version);
        }
    }
    
    ASSERT_TRUE(foundTool1);
    ASSERT_TRUE(foundTool2);
    
    PackageMetadataTestHelper::cleanupTempDir(tempDir);
}

TEST(ManagerMigrationFromLegacyFormat) {
    auto tempDir = PackageMetadataTestHelper::createTempDir();
    
    // Create legacy package files in root config directory
    auto legacyFile1 = tempDir / "legacy-tool1.json";
    auto legacyFile2 = tempDir / "legacy-tool2.json";
    
    PackageMetadataTestHelper::createJsonFile(legacyFile1, PackageMetadataTestHelper::createLegacyJson());
    PackageMetadataTestHelper::createJsonFile(legacyFile2, nlohmann::json{
        {"name", "legacy-tool2"},
        {"version", "2023.1202.1100"},
        {"hash", "legacy789hash012"},
        {"arch", "aarch64"}
    });
    
    PackageMetadataManager manager(tempDir);
    
    // Find legacy files
    auto legacyFiles = manager.findLegacyPackageFiles();
    ASSERT_EQ(2, legacyFiles.size());
    
    // Perform migration
    bool migrationResult = manager.migrateFromLegacyFormat();
    ASSERT_TRUE(migrationResult);
    
    // Verify packages directory was created
    ASSERT_TRUE(std::filesystem::exists(manager.getPackagesDirectory()));
    
    // Verify packages were migrated
    ASSERT_TRUE(manager.packageExists("legacy-tool1"));
    ASSERT_TRUE(manager.packageExists("legacy-tool2"));
    
    // Verify metadata has server information
    PackageMetadata migrated1 = manager.loadPackageMetadata("legacy-tool1");
    ASSERT_STR_EQ("getpkg.xyz", migrated1.sourceServer);
    ASSERT_NOT_EMPTY(migrated1.installDate);
    
    PackageMetadataTestHelper::cleanupTempDir(tempDir);
}

TEST(ManagerValidateAllPackageMetadata) {
    auto tempDir = PackageMetadataTestHelper::createTempDir();
    
    PackageMetadataManager manager(tempDir);
    manager.ensurePackagesDirectory();
    
    // Save valid and invalid metadata
    PackageMetadata validMetadata = PackageMetadataTestHelper::createValidMetadata();
    PackageMetadata invalidMetadata("", "", "", "", ""); // All empty - invalid
    
    manager.savePackageMetadata(validMetadata);
    
    // Manually create invalid metadata file
    auto invalidFile = manager.getPackagesDirectory() / "invalid-tool.json";
    PackageMetadataTestHelper::createJsonFile(invalidFile, PackageMetadataTestHelper::createCorruptedJson());
    
    bool validationResult = manager.validateAllPackageMetadata();
    ASSERT_FALSE(validationResult); // Should fail due to invalid metadata
    
    PackageMetadataTestHelper::cleanupTempDir(tempDir);
}

TEST(ManagerCleanupInvalidMetadata) {
    auto tempDir = PackageMetadataTestHelper::createTempDir();
    
    PackageMetadataManager manager(tempDir);
    manager.ensurePackagesDirectory();
    
    // Save valid metadata
    PackageMetadata validMetadata = PackageMetadataTestHelper::createValidMetadata();
    manager.savePackageMetadata(validMetadata);
    
    // Create invalid metadata file
    auto invalidFile = manager.getPackagesDirectory() / "invalid-tool.json";
    PackageMetadataTestHelper::createJsonFile(invalidFile, PackageMetadataTestHelper::createCorruptedJson());
    
    // Should have 2 files initially
    auto packagesBefore = manager.listInstalledPackages();
    ASSERT_EQ(2, packagesBefore.size());
    
    // Cleanup invalid metadata
    int cleanedCount = manager.cleanupInvalidMetadata();
    ASSERT_EQ(1, cleanedCount);
    
    // Should have 1 file after cleanup
    auto packagesAfter = manager.listInstalledPackages();
    ASSERT_EQ(1, packagesAfter.size());
    ASSERT_STR_EQ("test-tool", packagesAfter[0]);
    
    PackageMetadataTestHelper::cleanupTempDir(tempDir);
}

// Registration function
void registerPackageMetadataTests() {
    register_PackageMetadata_DefaultConstructor();
    register_PackageMetadata_ParameterizedConstructor();
    register_PackageMetadata_JsonSerialization();
    register_PackageMetadata_JsonDeserialization();
    register_PackageMetadata_LegacyJsonMigration();
    register_PackageMetadata_LegacyJsonMigrationWithCustomServer();
    register_PackageMetadata_ValidationValid();
    register_PackageMetadata_ValidationInvalidName();
    register_PackageMetadata_ValidationInvalidVersion();
    register_PackageMetadata_ValidationInvalidHash();
    register_PackageMetadata_ValidationInvalidArch();
    register_PackageMetadata_ValidationInvalidServer();
    register_PackageMetadata_FileOperationsSaveAndLoad();
    register_PackageMetadata_FileOperationsInvalidPath();
    register_PackageMetadata_NeedsUpdateComparison();
    register_PackageMetadata_TimestampGeneration();
    register_PackageMetadata_ManagerDefaultConstructor();
    register_PackageMetadata_ManagerCustomConstructor();
    register_PackageMetadata_ManagerEnsurePackagesDirectory();
    register_PackageMetadata_ManagerSaveAndLoadPackageMetadata();
    register_PackageMetadata_ManagerRemovePackageMetadata();
    register_PackageMetadata_ManagerListInstalledPackages();
    register_PackageMetadata_ManagerGetAllPackageMetadata();
    register_PackageMetadata_ManagerMigrationFromLegacyFormat();
    register_PackageMetadata_ManagerValidateAllPackageMetadata();
    register_PackageMetadata_ManagerCleanupInvalidMetadata();
}