#include "test_framework.hpp"
#include "ServerManager.hpp"
#include <filesystem>
#include <fstream>
#include <iostream>

// Test helper class for ServerManager testing
class ServerManagerTestHelper {
public:
    static std::filesystem::path createTempConfigDir() {
        auto tempDir = std::filesystem::temp_directory_path() / "getpkg_test" / std::to_string(std::time(nullptr));
        std::filesystem::create_directories(tempDir);
        return tempDir;
    }
    
    static void cleanupTempDir(const std::filesystem::path& dir) {
        if (std::filesystem::exists(dir)) {
            std::filesystem::remove_all(dir);
        }
    }
    
    static void createLegacyTokenFile(const std::filesystem::path& configDir, const std::string& token) {
        auto legacyDir = configDir / "getpkg.xyz";
        std::filesystem::create_directories(legacyDir);
        
        std::ofstream tokenFile(legacyDir / "write_token.txt");
        tokenFile << token;
        tokenFile.close();
    }
    
    static void createCorruptedConfigFile(const std::filesystem::path& configDir) {
        std::ofstream configFile(configDir / "servers.json");
        configFile << "{ invalid json content";
        configFile.close();
    }
};

TEST(ServerManager, DefaultConfiguration) {
    auto tempDir = ServerManagerTestHelper::createTempConfigDir();
    
    // Set environment variable to use temp directory
    setenv("HOME", tempDir.c_str(), 1);
    
    ServerManager manager;
    manager.ensureDefaultConfiguration();
    
    auto servers = manager.getServers();
    ASSERT_EQ(1, servers.size());
    ASSERT_STR_EQ("getpkg.xyz", servers[0]);
    ASSERT_STR_EQ("getpkg.xyz", manager.getDefaultServer());
    
    ServerManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(ServerManager, AddValidServer) {
    auto tempDir = ServerManagerTestHelper::createTempConfigDir();
    setenv("HOME", tempDir.c_str(), 1);
    
    ServerManager manager;
    manager.ensureDefaultConfiguration();
    
    auto result = manager.addServer("packages.example.com");
    ASSERT_EQ(ServerManagerError::None, result);
    
    auto servers = manager.getServers();
    ASSERT_EQ(2, servers.size());
    ASSERT_STR_EQ("getpkg.xyz", servers[0]);
    ASSERT_STR_EQ("packages.example.com", servers[1]);
    
    ServerManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(ServerManager, AddInvalidServer) {
    auto tempDir = ServerManagerTestHelper::createTempConfigDir();
    setenv("HOME", tempDir.c_str(), 1);
    
    ServerManager manager;
    manager.ensureDefaultConfiguration();
    
    // Test invalid URLs
    auto result1 = manager.addServer("not-a-url");
    ASSERT_EQ(ServerManagerError::InvalidUrl, result1);
    
    auto result2 = manager.addServer("");
    ASSERT_EQ(ServerManagerError::InvalidUrl, result2);
    
    auto result3 = manager.addServer("ftp://invalid-protocol.com");
    ASSERT_EQ(ServerManagerError::InvalidUrl, result3);
    
    // Should still have only default server
    auto servers = manager.getServers();
    ASSERT_EQ(1, servers.size());
    
    ServerManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(ServerManager, AddDuplicateServer) {
    auto tempDir = ServerManagerTestHelper::createTempConfigDir();
    setenv("HOME", tempDir.c_str(), 1);
    
    ServerManager manager;
    manager.ensureDefaultConfiguration();
    
    auto result1 = manager.addServer("packages.example.com");
    ASSERT_EQ(ServerManagerError::None, result1);
    
    auto result2 = manager.addServer("packages.example.com");
    ASSERT_EQ(ServerManagerError::ServerAlreadyExists, result2);
    
    auto servers = manager.getServers();
    ASSERT_EQ(2, servers.size()); // Should not add duplicate
    
    ServerManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(RemoveServer) {
    auto tempDir = ServerManagerTestHelper::createTempConfigDir();
    setenv("HOME", tempDir.c_str(), 1);
    
    ServerManager manager;
    manager.ensureDefaultConfiguration();
    manager.addServer("packages.example.com");
    manager.addServer("test.server.com");
    
    auto result = manager.removeServer("packages.example.com");
    ASSERT_EQ(ServerManagerError::None, result);
    
    auto servers = manager.getServers();
    ASSERT_EQ(2, servers.size());
    ASSERT_STR_EQ("getpkg.xyz", servers[0]);
    ASSERT_STR_EQ("test.server.com", servers[1]);
    
    ServerManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(RemoveNonExistentServer) {
    auto tempDir = ServerManagerTestHelper::createTempConfigDir();
    setenv("HOME", tempDir.c_str(), 1);
    
    ServerManager manager;
    manager.ensureDefaultConfiguration();
    
    auto result = manager.removeServer("nonexistent.server.com");
    ASSERT_EQ(ServerManagerError::ServerNotFound, result);
    
    ServerManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(RemoveLastServer) {
    auto tempDir = ServerManagerTestHelper::createTempConfigDir();
    setenv("HOME", tempDir.c_str(), 1);
    
    ServerManager manager;
    manager.ensureDefaultConfiguration();
    
    auto result = manager.removeServer("getpkg.xyz");
    ASSERT_EQ(ServerManagerError::LastServerRemoval, result);
    
    auto servers = manager.getServers();
    ASSERT_EQ(1, servers.size()); // Should still have the server
    
    ServerManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(WriteTokenManagement) {
    auto tempDir = ServerManagerTestHelper::createTempConfigDir();
    setenv("HOME", tempDir.c_str(), 1);
    
    ServerManager manager;
    manager.ensureDefaultConfiguration();
    manager.addServer("packages.example.com");
    
    // Set write token
    auto result = manager.setWriteToken("packages.example.com", "test-token-123");
    ASSERT_EQ(ServerManagerError::None, result);
    
    // Verify token
    ASSERT_TRUE(manager.hasWriteToken("packages.example.com"));
    ASSERT_STR_EQ("test-token-123", manager.getWriteToken("packages.example.com"));
    
    // Test servers with tokens
    auto serversWithTokens = manager.getServersWithTokens();
    ASSERT_EQ(1, serversWithTokens.size());
    ASSERT_STR_EQ("packages.example.com", serversWithTokens[0]);
    
    ServerManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(DefaultPublishServer) {
    auto tempDir = ServerManagerTestHelper::createTempConfigDir();
    setenv("HOME", tempDir.c_str(), 1);
    
    ServerManager manager;
    manager.ensureDefaultConfiguration();
    manager.addServer("packages.example.com");
    manager.addServer("test.server.com");
    
    // No tokens initially
    ASSERT_STR_EQ("", manager.getDefaultPublishServer());
    
    // Add token to second server
    manager.setWriteToken("test.server.com", "token2");
    ASSERT_STR_EQ("test.server.com", manager.getDefaultPublishServer());
    
    // Add token to first server - should become default
    manager.setWriteToken("getpkg.xyz", "token1");
    ASSERT_STR_EQ("getpkg.xyz", manager.getDefaultPublishServer());
    
    ServerManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(ConfigurationPersistence) {
    auto tempDir = ServerManagerTestHelper::createTempConfigDir();
    setenv("HOME", tempDir.c_str(), 1);
    
    {
        ServerManager manager1;
        manager1.ensureDefaultConfiguration();
        manager1.addServer("packages.example.com");
        manager1.setWriteToken("packages.example.com", "test-token");
        manager1.saveConfiguration();
    }
    
    {
        ServerManager manager2;
        auto result = manager2.loadConfiguration();
        ASSERT_EQ(ServerManagerError::None, result);
        
        auto servers = manager2.getServers();
        ASSERT_EQ(2, servers.size());
        ASSERT_STR_EQ("getpkg.xyz", servers[0]);
        ASSERT_STR_EQ("packages.example.com", servers[1]);
        
        ASSERT_TRUE(manager2.hasWriteToken("packages.example.com"));
        ASSERT_STR_EQ("test-token", manager2.getWriteToken("packages.example.com"));
    }
    
    ServerManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(CorruptedConfigurationRecovery) {
    auto tempDir = ServerManagerTestHelper::createTempConfigDir();
    setenv("HOME", tempDir.c_str(), 1);
    
    // Create corrupted config file
    ServerManagerTestHelper::createCorruptedConfigFile(tempDir / ".config" / "getpkg");
    
    ServerManager manager;
    auto result = manager.loadConfiguration();
    
    // Should recover by creating default configuration
    ASSERT_EQ(ServerManagerError::None, result);
    
    auto servers = manager.getServers();
    ASSERT_EQ(1, servers.size());
    ASSERT_STR_EQ("getpkg.xyz", servers[0]);
    
    ServerManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(LegacyMigration) {
    auto tempDir = ServerManagerTestHelper::createTempConfigDir();
    setenv("HOME", tempDir.c_str(), 1);
    
    auto configDir = tempDir / ".config" / "getpkg";
    std::filesystem::create_directories(configDir);
    
    // Create legacy token file
    ServerManagerTestHelper::createLegacyTokenFile(configDir, "legacy-token-123");
    
    ServerManager manager;
    bool migrated = manager.migrateFromLegacy();
    ASSERT_TRUE(migrated);
    
    // Verify migration
    ASSERT_TRUE(manager.hasWriteToken("getpkg.xyz"));
    ASSERT_STR_EQ("legacy-token-123", manager.getWriteToken("getpkg.xyz"));
    
    ServerManagerTestHelper::cleanupTempDir(tempDir);
}

TEST(ErrorMessages) {
    ServerManager manager;
    
    ASSERT_NOT_EMPTY(manager.getErrorMessage(ServerManagerError::InvalidUrl));
    ASSERT_NOT_EMPTY(manager.getErrorMessage(ServerManagerError::ServerNotFound));
    ASSERT_NOT_EMPTY(manager.getErrorMessage(ServerManagerError::ServerAlreadyExists));
    ASSERT_NOT_EMPTY(manager.getErrorMessage(ServerManagerError::LastServerRemoval));
}

// Registration function
void registerServerManagerTests() {
    register_ServerManager_DefaultConfiguration();
    register_ServerManager_AddValidServer();
    register_ServerManager_AddInvalidServer();
    register_ServerManager_AddDuplicateServer();
    register_ServerManager_RemoveServer();
    register_ServerManager_RemoveNonExistentServer();
    register_ServerManager_RemoveLastServer();
    register_ServerManager_WriteTokenManagement();
    register_ServerManager_DefaultPublishServer();
    register_ServerManager_ConfigurationPersistence();
    register_ServerManager_CorruptedConfigurationRecovery();
    register_ServerManager_LegacyMigration();
    register_ServerManager_ErrorMessages();
}